--/Example 12

--/ Example of using JOINS between tables/views

--/ We're all so good at designing our database tables - following those Normalisation steps - that in most query 
--/ situations, the data we want in our result set is not in a single table/view. Therefore we have to create queries
--/ that can return data from multiple tables/views. To do this we use the JOIN command.

--/ we see the following explanation in the help file

--/ FROM first_table join_type second_table [ON (join_condition)]

--/ join_type specifies what kind of join is performed: an inner, outer, or cross join. 
--/ join_condition defines the predicate to be evaluated for each pair of joined rows.

--/ As a first example of using joins, lets look at the Orders and Order Details tables and how they relate to each other
--/ and how we use the relationship between them to retrieve data from both tables

--/ As you can guess, the key field in the orders table is the OrderID field
--/ and in the Order Details table, the combination of OrderID and ProductID (not great but hey, I didn't design it!)

--/ although joins can be written to link tables between any like type fields, it is easier to start off looking at joins
--/ with fields 'designed' for the job and in the following examples I will be using the OrderID field as the linking field

--/ a first JOIN query - a simple INNER JOIN

SELECT * FROM Orders 
	INNER JOIN [Order Details] ON orders.orderID = [Order Details].orderID 

--/ If we run this query and look at the results we now have all the rows from the [Order Details] and for each of 
--/ these rows we have the corresponding order details from the orders table.
--/ In situations like this, where the table name requires [] or where the table name is very long, it may be easier
--/ to add a table alias as in the following example

SELECT * FROM Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 

--/ we can now use all the other tools we have used to produce more sophisticated (OK not that sophisticated) queries!
--/ you may notice from the result set that there is no product information other than the productID so let's add this
--/ information by joining the relevant product table 

SELECT * FROM Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 
	JOIN Products p ON od.ProductID = p.ProductID
	
--/ Let's limit the fields retrieved rather than using the * parameter

SELECT 
	o.orderID, o.CustomerID, CONVERT(VARCHAR(10), o.OrderDate, 103) AS OrderDateAsString,
	od.Quantity, od.UnitPrice, p.ProductName
FROM 	Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 
	JOIN Products p ON od.ProductID = p.ProductID


--/ Note there is no need to specify the table name prefix to a column as long as that column is UNIQUE amongst all
--/ the columns within the tables in the statement

--/ Let's expand this further by adding tables relating to customers and to suppliers

SELECT 
	o.orderID, o.CustomerID, CONVERT(VARCHAR(10), o.OrderDate, 103) AS OrderDateAsString,
	od.Quantity, od.UnitPrice, 
	p.ProductName,
	c.CompanyName AS CustomerCompanyName,
	s.CompanyName AS SupplierCompanyName
FROM 	Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 
	JOIN Products p ON od.ProductID = p.ProductID
	JOIN Suppliers s ON p.supplierID = s.supplierID
	JOIN Customers c ON c.CustomerID = o.CustomerID

--/ Note that it is not important what order tables are listed in the FROM clause as long as the 'join type' is specified
--/ correctly for the way in which the tables/views should be joined. Equally the ON clause details are not order specific

SELECT 
	o.orderID, o.CustomerID, CONVERT(VARCHAR(10), o.OrderDate, 103) AS OrderDateAsString,
	od.Quantity, od.UnitPrice, 
	p.ProductName,
	c.CompanyName AS CustomerCompanyName,
	s.CompanyName AS SupplierCompanyName
FROM 	Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 
	JOIN Customers c ON c.CustomerID = o.CustomerID
	JOIN Products p ON od.ProductID = p.ProductID
	JOIN Suppliers s ON p.supplierID = s.supplierID

--/ However, do ensure that you specify a table/view (and relevant alias) prior to referring to it in a join predicate
--/ So this will NOT work as the 2nd JOIN (JOIN Suppliers s ON p.supplierID = s.supplierID) specifies a table p
--/ which at this point in the query has not yet been specified

SELECT 
	o.orderID, o.CustomerID, CONVERT(VARCHAR(10), o.OrderDate, 103) AS OrderDateAsString,
	od.Quantity, od.UnitPrice, 
	p.ProductName,
	c.CompanyName AS CustomerCompanyName,
	s.CompanyName AS SupplierCompanyName
FROM 	Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 
	JOIN Suppliers s ON p.supplierID = s.supplierID
	JOIN Customers c ON c.CustomerID = o.CustomerID
	JOIN Products p ON od.ProductID = p.ProductID

--/ also note that these two JOINS produce the same result

SELECT 
	o.orderID, o.CustomerID, CONVERT(VARCHAR(10), o.OrderDate, 103) AS OrderDateAsString,
	od.Quantity, od.UnitPrice, p.ProductName
FROM 	Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 
	RIGHT JOIN Products p ON od.ProductID = p.ProductID

SELECT 
	o.orderID, o.CustomerID, CONVERT(VARCHAR(10), o.OrderDate, 103) AS OrderDateAsString,
	od.Quantity, od.UnitPrice, p.ProductName
FROM 	[Order Details] od
	LEFT JOIN Products p ON od.ProductID = p.ProductID
	JOIN Orders o ON o.orderID = od.orderID 

